package sample.parser;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.lightysoft.logmx.business.ParsedEntry;
import com.lightysoft.logmx.mgr.LogFileParser;

/**
 * Sample LogMX Parser able to parse Websphere log files with multi-line support and relative & absolute Dates support.<BR/>
 * 
 * Here is an example of log file suitable for this parser:<BR/>
 * 
 * [3/17/07 14:30:10:270 EDT] 0000000a ManagerAdmin  I   TRAS0017I: The startup trace state is *=info.
 * [3/17/07 22:35:47:532 EDT] 00000115 CMAdminServic E com.ibm.lotus.search.cm.admin.imp.
 *  CMAdminService createCollection EJPJK0007E: Failed to create searchable, 
 *  DefaultContentModelCollection already exists for host localhost
 * [3/17/07 22:44:25:677 EDT] 0000008e SRTServletRes W   WARNING: Cannot set header. 
 *   Response already committed.
 */
public class WebsphereLogParser extends LogFileParser {
    /** Current parsed log entry */
    private ParsedEntry entry = null;

    /** Entry date format */
    private final static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat(
        "M/d/yy HH:mm:ss:SSS z");

    /** Pattern for entry begin */
    private final static Pattern ENTRY_BEGIN_PATTERN = Pattern
        .compile("^\\[(\\d{1,2}/\\d{1,2}/\\d{2} \\d{2}:\\d{2}:\\d{2}:\\d{1,3} [^\\]]*?)\\] (.{8}) (.{13}) (\\w) (.*?)$");

    /** Buffer for Entry message (improves performance for multi-lines entries)  */
    private StringBuilder entryMsgBuffer = null;


    /** 
     * Returns the name of this parser
     * @see com.lightysoft.logmx.mgr.LogFileParser#getParserName()
     */
    public String getParserName() {
        return "Websphere Log Parser";
    }

    /**
     * Returns the supported file type for this parser
     * @see com.lightysoft.logmx.mgr.LogFileParser#getSupportedFileType()
     */
    public String getSupportedFileType() {
        return "Websphere log files";
    }

    /**
     * Process the new line of text read from file 
     * @see com.lightysoft.logmx.mgr.LogFileParser#parseLine(java.lang.String)
     */
    protected void parseLine(String line) throws Exception {
        // If end of file, records last entry if necessary, and exits
        if (line == null) {
            recordPreviousEntryIfExists();
            return;
        }

        Matcher matcher = ENTRY_BEGIN_PATTERN.matcher(line);
        if (matcher.matches()) {
            // Record previous found entry if exists, then create a new one
            prepareNewEntry();

            entry.setDate(matcher.group(1));
            entryMsgBuffer.append("[msg id=").append(matcher.group(2)).append(']').append(' ')
                .append(matcher.group(5));
            entry.setEmitter(matcher.group(3));

            final char logLevelChar = matcher.group(4).charAt(0);
            if (logLevelChar == 'I') {
                entry.setLevel("INFO");
            } else if (logLevelChar == 'W') {
                entry.setLevel("WARNING");
            } else if (logLevelChar == 'E' || logLevelChar == 'R') {
                entry.setLevel("ERROR");
            } // else, the default Log Level will be set by LogMX
        } else if (entry != null) {
            entryMsgBuffer.append('\n').append(line); // appends this line to previous entry's text
        }
    }

    /**
     * Returns the relative timestamp of given entry 
     * @see com.lightysoft.logmx.mgr.LogFileParser#getRelativeEntryDate(com.lightysoft.logmx.business.ParsedEntry)
     */
    public Date getRelativeEntryDate(ParsedEntry pEntry) throws Exception {
        return getAbsoluteEntryDate(pEntry);
    }

    /**
     * Returns the Date object for the given entry 
     * @see com.lightysoft.logmx.mgr.LogFileParser#getAbsoluteEntryDate(com.lightysoft.logmx.business.ParsedEntry)
     */
    public Date getAbsoluteEntryDate(ParsedEntry pEntry) throws Exception {
        return DATE_FORMAT.parse(pEntry.getDate());
    }

    /**
     * Send to LogMX the current parsed log entry
     * @throws Exception
     */
    private void recordPreviousEntryIfExists() throws Exception {
        if (entry != null) {
            entry.setMessage(entryMsgBuffer.toString());
            addEntry(entry);
        }
    }

    /**
     * Send to LogMX the current parsed log entry, then create a new one
     * @throws Exception
     */
    private void prepareNewEntry() throws Exception {
        recordPreviousEntryIfExists();
        entry = createNewEntry();
        entryMsgBuffer = new StringBuilder(80);
    }
}
